﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.Collections.ObjectModel
Public Class Form1
   Private moLine As LineShape
   Private moPolygon As PolygonShape
   Private moMap As WinformsMap
   Private moShapefile As MyShapefile
   Private moInMemoryFeatureLayer As New InMemoryFeatureLayer
   Private WithEvents moTrackLayer As TrackInteractiveOverlay
   'Private WithEvents moEditOverlay As MyEditOverlay
   Private msKey As String
   Private moEditFeatures As New Collection(Of Feature)
   Public Const cdMICRON As Double = 0.000001
   Private msInstructions As String =
      "1. Select features by clicking with the mouse." & ControlChars.CrLf &
      "2. Delete features using the delete key." & ControlChars.CrLf &
      "3. Notice that an exception is thrown when you delete the last feature." & ControlChars.CrLf

   Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
      Dim oLayerOverlay As New LayerOverlay
      Dim oSelectionOverlay As New LayerOverlay

      Me.TextBox1.Text = msInstructions
      moMap = WinformsMap1
      moMap.MapUnit = GeographyUnit.Meter
      moMap.MinimumScale = 0
      ShapeFileFeatureLayer.Rebuild("..\..\Data\TestPoly.shp")
      moShapefile = New MyShapefile("..\..\Data\TestPoly.shp", ShapeFileReadWriteMode.ReadWrite)

      moShapefile.Name = "TestPoly"
      oLayerOverlay.Layers.Add("Shapefile", moShapefile)

      Dim oAreaStyle As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.GeographicColors.InnerState, 1)
      moShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oAreaStyle)
      moShapefile.AreaSymbol = oAreaStyle

      Dim oTextStyle As TextStyle = TextStyles.Country1("Name")
      oTextStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels
      oTextStyle.OverlappingRule = LabelOverlappingRule.AllowOverlapping
      oTextStyle.LabelAllPolygonParts = True
      moShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oTextStyle)
      moShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

      moShapefile.TextSymbol = oTextStyle
      moShapefile.Open()

      WinformsMap1.Overlays.Add(oLayerOverlay)


      moTrackLayer = WinformsMap1.TrackOverlay

      WinformsMap1.CurrentExtent = moShapefile.GetBoundingBox
      WinformsMap1.Refresh()
   End Sub

   Private Sub WinformsMap1_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles WinformsMap1.KeyUp
      Dim oSelectionLayer As InMemoryFeatureLayer = WinformsMap1.EditOverlay.EditShapesLayer

      If oSelectionLayer.InternalFeatures.Count > 0 Then
         If Not moShapefile.IsOpen Then
            moShapefile.Close()
         End If
         moShapefile.EditTools.BeginTransaction()
         For Each oFeature As Feature In oSelectionLayer.InternalFeatures
            moShapefile.EditTools.Delete(oFeature.Id)
         Next oFeature
         moShapefile.EditTools.CommitTransaction()
         moShapefile.Close()
         ShapeFileFeatureLayer.BuildIndexFile(moShapefile.ShapePathFileName)
         oSelectionLayer.InternalFeatures.Clear()
         WinformsMap1.Refresh()
      End If

   End Sub


   Private Sub WinformsMap1_MapClick(ByVal sender As Object, ByVal e As ThinkGeo.MapSuite.DesktopEdition.MapClickWinformsMapEventArgs) Handles WinformsMap1.MapClick
      Dim oFeatures As Collection(Of Feature)
      If e.MouseButton = MapMouseButton.Left Then

         If msKey <> String.Empty Then
            With WinformsMap1.EditOverlay.EditShapesLayer
               .Open()
               .EditTools.IsTransactionLive = True
               .EditTools.BeginTransaction()
               .EditTools.Delete(msKey)
               .EditTools.CommitTransaction()
            End With
         End If

         '***** Select a feature
         oFeatures = moShapefile.QueryTools.GetFeaturesContaining(New PointShape(e.WorldX, e.WorldY), ReturningColumnsType.AllColumns)
         If oFeatures.Count > 0 Then
            msKey = oFeatures(0).Id
            With WinformsMap1.EditOverlay.EditShapesLayer
               .Open()
               .EditTools.IsTransactionLive = True
               .EditTools.BeginTransaction()
               .EditTools.Add(oFeatures(0))
               .EditTools.CommitTransaction()
            End With
         End If

         WinformsMap1.Refresh()
      End If
   End Sub

   Private Sub moTrackLayer_TrackEnded(ByVal sender As Object, ByVal e As ThinkGeo.MapSuite.DesktopEdition.TrackEndedTrackInteractiveOverlayEventArgs) Handles moTrackLayer.TrackEnded
      mAddNewFeature(e.TrackShape)
      moTrackLayer.TrackShapeLayer.InternalFeatures.Clear()
      WinformsMap1.Refresh()
   End Sub
   Private Sub mAddNewFeature(ByVal oShape As BaseShape)
      Dim oPoly As PolygonShape = CType(oShape, PolygonShape)
      Dim oCols As New Dictionary(Of String, String)

      If Not moShapefile.IsOpen Then
         moShapefile.Open()
      End If

      '***** Create a name foe the new feature
      Dim iNumFeatures As Integer = moShapefile.QueryTools.GetCount() + 1
      Dim sNewName As String = moShapefile.Name & " - " & iNumFeatures.ToString
      oCols.Add("Name", sNewName)
      oCols.Add("NewColumn", Guid.NewGuid.ToString)
      Dim oNewFeature As New Feature(oPoly, oCols)

      With moShapefile.EditTools
         If Not .IsInTransaction Then
            .IsTransactionLive = True
            .BeginTransaction()
         End If
         .Add(oNewFeature)
      End With
      moShapefile.Close()

      ShapeFileFeatureLayer.BuildIndexFile(moShapefile.ShapePathFileName, BuildIndexMode.Rebuild)

   End Sub

   Private Sub mCommitTransaction()
      If Not moShapefile.IsOpen Then
         moShapefile.Open()
      End If

      If moShapefile.EditTools.IsInTransaction Then
         moShapefile.EditTools.CommitTransaction()
         moShapefile.Close()
         WinformsMap1.Refresh()
      End If

   End Sub
End Class
